Exception: Scarpe::Error
- Inherits:
-
StandardError
- Object
- StandardError
- Scarpe::Error
- Defined in:
- lacci/lib/lacci/scarpe_core.rb
Overview
The base error class for Scarpe errors, but not necessarily Shoes::Errors
The base error class for Scarpe errors, but not necessarily Shoes::Errors