Class: Shoes::DocumentRoot
- Defined in:
- lacci/lib/shoes/drawables/document_root.rb
Constant Summary
Constants included from Log
Log::DEFAULT_COMPONENT, Log::DEFAULT_DEBUG_LOG_CONFIG, Log::DEFAULT_LOG_CONFIG
Instance Attribute Summary
Attributes inherited from Slot
Attributes inherited from Drawable
#debug_id, #destroyed, #parent
Attributes inherited from Linkable
Instance Method Summary collapse
-
#initialize ⇒ DocumentRoot
constructor
A new instance of DocumentRoot.
-
#inspect ⇒ Object
The default inspect string can be absolutely huge in console output, and it's frequently printed.
Methods included from Border
Methods included from Background
Methods inherited from Slot
#add_child, #append, #clear, #contents, #method_missing, #remove_child, #respond_to_missing?
Methods inherited from Drawable
allocate_drawable_id, #app, #banner, #caption, #destroy, #download, drawable_by_id, drawable_class_by_name, dsl_name, #event, get_shoes_events, #hide, #inscription, is_widget_class?, #method_missing, register_drawable_id, #respond_to_missing?, #set_parent, shoes_events, shoes_style, shoes_style_hashes, shoes_style_name?, shoes_style_names, #shoes_style_values, shoes_styles, #show, #style, #subtitle, #tagline, #title, #toggle, unregister_drawable_id, validate_as
Methods included from Colors
Methods included from Log
configure_logger, #log_init, logger
Methods inherited from Linkable
#bind_shoes_event, #send_self_event, #send_shoes_event, #unsub_all_shoes_events, #unsub_shoes_event
Constructor Details
#initialize ⇒ DocumentRoot
Returns a new instance of DocumentRoot.
7 8 9 10 11 12 13 |
# File 'lacci/lib/shoes/drawables/document_root.rb', line 7 def initialize @height = "100%" @width = @margin = @padding = nil @options = {} super end |
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class Shoes::Slot
Instance Method Details
#inspect ⇒ Object
The default inspect string can be absolutely huge in console output, and it's frequently printed.
16 17 18 |
# File 'lacci/lib/shoes/drawables/document_root.rb', line 16 def inspect "<Shoes::DocumentRoot>" end |