Class: Shoes::DocumentRoot
- Defined in:
- lacci/lib/shoes/widgets/document_root.rb
Constant Summary
Constants included from Log
Log::DEFAULT_COMPONENT, Log::DEFAULT_DEBUG_LOG_CONFIG, Log::DEFAULT_LOG_CONFIG
Instance Attribute Summary
Attributes inherited from Slot
Attributes inherited from Widget
Attributes inherited from Linkable
Instance Method Summary collapse
-
#initialize ⇒ DocumentRoot
constructor
A new instance of DocumentRoot.
-
#inspect ⇒ Object
The default inspect string can be absolutely huge in console output, and it's frequently printed.
Methods included from Border
Methods included from Background
Methods inherited from Slot
#add_child, #app, #append, #clear, #contents, #remove_child
Methods inherited from Widget
#banner, #caption, #destroy, display_properties, display_property, display_property_name?, display_property_names, #display_property_values, #download, dsl_name, #hide, #inscription, #method_missing, #respond_to_missing?, #set_parent, #show, #subtitle, #tagline, #title, #toggle, widget_class_by_name
Methods included from Colors
Methods included from Log
configure_logger, #log_init, logger
Methods inherited from Linkable
#bind_shoes_event, #send_self_event, #send_shoes_event, #unsub_shoes_event
Constructor Details
#initialize ⇒ DocumentRoot
Returns a new instance of DocumentRoot.
5 6 7 8 9 10 11 12 13 |
# File 'lacci/lib/shoes/widgets/document_root.rb', line 5 def initialize @height = "100%" @width = @margin = @padding = nil @options = {} super end |
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class Shoes::Widget
Instance Method Details
#inspect ⇒ Object
The default inspect string can be absolutely huge in console output, and it's frequently printed.
16 17 18 |
# File 'lacci/lib/shoes/widgets/document_root.rb', line 16 def inspect "<Shoes::DocumentRoot>" end |