Class: Shoes::TextDrawable
- Defined in:
- lacci/lib/shoes/drawables/text_drawable.rb
Overview
TextDrawable is the parent class of various classes of text that can go inside a para. This includes normal text, but also links, italic text, bold text, etc.
Direct Known Subclasses
Constant Summary
Constants included from Log
Log::DEFAULT_COMPONENT, Log::DEFAULT_DEBUG_LOG_CONFIG, Log::DEFAULT_LOG_CONFIG
Instance Attribute Summary
Attributes inherited from Drawable
#debug_id, #destroyed, #parent
Attributes inherited from Linkable
Class Method Summary collapse
-
.inherited(subclass) ⇒ Object
rubocop:disable Lint/MissingSuper.
Methods inherited from Drawable
allocate_drawable_id, #app, #banner, #caption, #destroy, #download, drawable_by_id, drawable_class_by_name, dsl_name, #event, get_shoes_events, #hide, #initialize, #inscription, #inspect, is_widget_class?, #method_missing, register_drawable_id, #respond_to_missing?, #set_parent, shoes_events, shoes_style, shoes_style_hashes, shoes_style_name?, shoes_style_names, #shoes_style_values, shoes_styles, #show, #style, #subtitle, #tagline, #title, #toggle, unregister_drawable_id, validate_as
Methods included from Colors
Methods included from Log
configure_logger, #log_init, logger
Methods inherited from Linkable
#bind_shoes_event, #initialize, #send_self_event, #send_shoes_event, #unsub_all_shoes_events, #unsub_shoes_event
Constructor Details
This class inherits a constructor from Shoes::Drawable
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class Shoes::Drawable
Class Method Details
.inherited(subclass) ⇒ Object
rubocop:disable Lint/MissingSuper
10 11 12 13 14 15 16 |
# File 'lacci/lib/shoes/drawables/text_drawable.rb', line 10 def inherited(subclass) Shoes::Drawable.drawable_classes ||= [] Shoes::Drawable.drawable_classes << subclass Shoes::Drawable.drawable_default_styles ||= {} Shoes::Drawable.drawable_default_styles[subclass] = {} end |